import RUBY_IMG from '../assets/images/tools/ruby.svg'
import RUBY_ON_RAILS_IMG from '../assets/images/tools/ruby_on_rails.svg'
import PHP_IMG from '../assets/images/tools/php.svg'
import SYMFONY_IMG from '../assets/images/tools/symfony.svg'
import SYLIUS_IMG from '../assets/images/tools/sylius.png'
import JAVASCRIPT_IMG from '../assets/images/tools/javascript.svg'
import TYPESCRIPT_IMG from '../assets/images/tools/typescript.svg'
import REACT_IMG from '../assets/images/tools/react.svg'
import HOTWIRE_IMG from '../assets/images/tools/hotwire.svg'
import CSS_IMG from '../assets/images/tools/css3.svg'
import SASS_IMG from '../assets/images/tools/sass.svg'
import TAILWIND_IMG from '../assets/images/tools/tailwindcss.svg'
import BOOTSTRAP_IMG from '../assets/images/tools/bootstrap.svg'
import DART_IMG from '../assets/images/tools/dart.svg'
import FLUTTER_IMG from '../assets/images/tools/flutter.svg'
import RUST_IMG from '../assets/images/tools/rust.svg'
import JAVA_IMG from '../assets/images/tools/java.svg'
import OPENGL_IMG from '../assets/images/tools/opengl.svg'
import LWJGL_IMG from '../assets/images/tools/lwjgl.svg'
import CSHARP_IMG from '../assets/images/tools/csharp.svg'
import UNITY3D_IMG from '../assets/images/tools/unity3d.svg'
import GODOTENGINE_IMG from '../assets/images/tools/godotengine.svg'
import WEBPACK_IMG from '../assets/images/tools/webpack.svg'
import VITE_IMG from '../assets/images/tools/vite.svg'
import REQUIREJS_IMG from '../assets/images/tools/requirejs.svg'
import STEAMAPI_IMG from '../assets/images/tools/steamapi.svg'
import MIDI_IMG from '../assets/images/tools/midi.svg'
import CORDOVA_IMG from '../assets/images/tools/cordova.svg'
import ELECTRON_IMG from '../assets/images/tools/electron.svg'

enum ToolType {
    LANGUAGE = 0b0001,
    FRAMEWORK = 0b0010,
    LIB_OR_PACKAGE = 0b0100,
    OTHER = 0b1000
}

enum ToolPlatform {
    MOBILE = 0b00001,
    WEB_FRONT = 0b00010,
    WEB_BACK = 0b00100,
    SYSTEM = 0b01000,
    DESKTOP = 0b10000
}

interface Tool {
    type: ToolType,
    name: string,
    version: string | null,
    description: string | null,
    icon: string | null,
    url: string,
    depend_tools: Tool[],
    developed_platforms: ToolPlatform[]
}

export default Tool

// RUBY

export const Ruby: Tool = {
    type: ToolType.LANGUAGE,
    name: 'Ruby',
    version: '2.7 - 3.2',
    icon: RUBY_IMG,
    description: null,
    url: 'https://www.ruby-lang.org/',
    depend_tools: [],
    developed_platforms: [ToolPlatform.MOBILE, ToolPlatform.WEB_BACK]
}

export const Rails: Tool = {
    type: ToolType.FRAMEWORK,
    name: 'Ruby on rails',
    version: '5.1 - 7.0',
    icon: RUBY_ON_RAILS_IMG,
    description: null,
    url: 'https://rubyonrails.org/',
    depend_tools: [Ruby],
    developed_platforms: [ToolPlatform.WEB_BACK]
}

// PHP

export const Php: Tool = {
    type: ToolType.LANGUAGE,
    name: 'PHP',
    icon: PHP_IMG,
    version: '5 et 7',
    description: null,
    url: 'https://www.php.net/',
    depend_tools: [],
    developed_platforms: [ToolPlatform.WEB_BACK]
}

export const Symfony: Tool = {
    type: ToolType.FRAMEWORK,
    name: 'Symfony',
    version: '4.0 - 5.0',
    icon: SYMFONY_IMG,
    description: null,
    url: 'https://symfony.com/',
    depend_tools: [Php],
    developed_platforms: [ToolPlatform.WEB_BACK]
}

export const Sylius: Tool = {
    type: ToolType.FRAMEWORK,
    name: 'Sylius',
    version: '1.6',
    icon: SYLIUS_IMG,
    description: null,
    url: 'https://symfony.com/',
    depend_tools: [Symfony],
    developed_platforms: [ToolPlatform.WEB_BACK]
}

// JS

export const Javascript: Tool = {
    type: ToolType.LANGUAGE,
    name: 'Javascript',
    version: null,
    icon: JAVASCRIPT_IMG,
    description: null,
    url: 'https://www.javascript.com/',
    depend_tools: [],
    developed_platforms: [ToolPlatform.WEB_BACK, ToolPlatform.WEB_FRONT]
}

export const Typescript: Tool = {
    type: ToolType.LANGUAGE,
    name: 'Typescript',
    version: null,
    icon: TYPESCRIPT_IMG,
    description: null,
    url: 'https://www.typescriptlang.org/',
    depend_tools: [],
    developed_platforms: Javascript.developed_platforms
}

export const React: Tool = {
    type: ToolType.FRAMEWORK,
    name: 'React',
    version: '14, 15, 16, 17, 18',
    icon: REACT_IMG,
    description: null,
    url: 'https://react.dev/',
    depend_tools: [Javascript, Typescript],
    developed_platforms: [ToolPlatform.WEB_FRONT]
}

export const Hotwire: Tool = {
    type: ToolType.FRAMEWORK,
    name: 'Hotwire',
    version: '2 - 3',
    icon: HOTWIRE_IMG,
    description: 'Turbo + Stimulus',
    url: 'https://stimulus.hotwired.dev/',
    depend_tools: [Javascript, Typescript, Rails],
    developed_platforms: [ToolPlatform.WEB_FRONT]
}

// CSS

export const CSS: Tool = {
    type: ToolType.LANGUAGE,
    name: 'CSS',
    version: null,
    icon: CSS_IMG,
    description: null,
    url: 'https://www.w3schools.com/css/',
    depend_tools: [],
    developed_platforms: [ToolPlatform.MOBILE, ToolPlatform.DESKTOP, ToolPlatform.WEB_FRONT]
}

export const SASS: Tool = {
    type: ToolType.LANGUAGE,
    name: 'SASS',
    version: null,
    icon: SASS_IMG,
    description: null,
    url: 'https://sass-lang.com/',
    depend_tools: [CSS],
    developed_platforms: CSS.developed_platforms
}

export const Tailwind: Tool = {
    type: ToolType.OTHER,
    name: 'Tailwind',
    version: null,
    icon: TAILWIND_IMG,
    description: null,
    url: 'https://tailwindcss.com/',
    depend_tools: [SASS],
    developed_platforms: SASS.developed_platforms
}

export const Bootstrap: Tool = {
    type: ToolType.OTHER,
    name: 'Bootstrap',
    version: null,
    icon: BOOTSTRAP_IMG,
    description: null,
    url: 'https://getbootstrap.com/',
    depend_tools: [CSS],
    developed_platforms: CSS.developed_platforms
}

// FLUTTER

export const Dart: Tool = {
    type: ToolType.LANGUAGE,
    name: 'Dart',
    version: '2 - 3',
    icon: DART_IMG,
    description: null,
    url: 'https://dart.dev/',
    depend_tools: [],
    developed_platforms: [ToolPlatform.MOBILE, ToolPlatform.DESKTOP]
}

export const Flutter: Tool = {
    type: ToolType.FRAMEWORK,
    name: 'Flutter',
    version: '2 - 3',
    icon: FLUTTER_IMG,
    description: null,
    url: 'https://flutter.dev/',
    depend_tools: [Dart],
    developed_platforms: Dart.developed_platforms
}

// RUST

export const Rust: Tool = {
    type: ToolType.LANGUAGE,
    name: 'Rust',
    version: 'Édition 2018, 2021',
    icon: RUST_IMG,
    description: null,
    url: 'https://www.rust-lang.org/',
    depend_tools: [],
    developed_platforms: [ToolPlatform.SYSTEM, ToolPlatform.WEB_FRONT, ToolPlatform.WEB_BACK]
}

// 3D

export const Java: Tool = {
    type: ToolType.LANGUAGE,
    name: 'Java',
    version: '5 - 8',
    icon: JAVA_IMG,
    description: null,
    url: 'https://www.java.com/',
    depend_tools: [],
    developed_platforms: [ToolPlatform.MOBILE, ToolPlatform.WEB_BACK, ToolPlatform.DESKTOP]
}

export const OpenGL: Tool = {
    type: ToolType.OTHER,
    name: 'OpenGL',
    version: '<= 4.0',
    icon: OPENGL_IMG,
    description: null,
    url: 'https://www.opengl.org/',
    depend_tools: [],
    developed_platforms: [ToolPlatform.DESKTOP]
}

export const LWJGL: Tool = {
    type: ToolType.LIB_OR_PACKAGE,
    name: 'LWJGL',
    version: '2 et 3',
    icon: LWJGL_IMG,
    description: null,
    url: 'https://www.opengl.org/',
    depend_tools: [Java, OpenGL],
    developed_platforms: [ToolPlatform.DESKTOP]
}

export const CSharp: Tool = {
    type: ToolType.LANGUAGE,
    name: 'C#',
    version: null,
    icon: CSHARP_IMG,
    description: null,
    url: 'https://www.microsoft.com',
    depend_tools: [],
    developed_platforms: [ToolPlatform.DESKTOP, ToolPlatform.MOBILE, ToolPlatform.WEB_FRONT]
}

export const Unity3D: Tool = {
    type: ToolType.OTHER,
    name: 'Unity3D',
    version: '2018-2019',
    icon: UNITY3D_IMG,
    description: null,
    url: 'https://unity.com/',
    depend_tools: [CSharp, OpenGL],
    developed_platforms: CSharp.developed_platforms
}

export const GodotEngine: Tool = {
    type: ToolType.OTHER,
    name: 'GodotEngine',
    version: '3.x',
    icon: GODOTENGINE_IMG,
    description: null,
    url: 'https://godotengine.org/',
    depend_tools: [CSharp, OpenGL],
    developed_platforms: CSharp.developed_platforms
}

// Builder

export const Webpack: Tool = {
    type: ToolType.OTHER,
    name: 'Webpack',
    version: null,
    icon: WEBPACK_IMG,
    description: null,
    url: 'https://webpack.js.org/',
    depend_tools: [],
    developed_platforms: CSS.developed_platforms
}

export const Vite: Tool = {
    type: ToolType.OTHER,
    name: 'Vite',
    version: null,
    icon: VITE_IMG,
    description: null,
    url: 'https://vitejs.dev/',
    depend_tools: [],
    developed_platforms: CSS.developed_platforms
}

export const RequireJS: Tool = {
    type: ToolType.OTHER,
    name: 'RequireJS',
    version: null,
    icon: REQUIREJS_IMG,
    description: null,
    url: 'https://requirejs.org/',
    depend_tools: [],
    developed_platforms: CSS.developed_platforms
}

// Other

export const SteamAPI: Tool = {
    type: ToolType.LIB_OR_PACKAGE,
    name: 'SteamAPI',
    version: null,
    icon: STEAMAPI_IMG,
    description: null,
    url: 'https://steamcommunity.com/dev',
    depend_tools: [],
    developed_platforms: Unity3D.developed_platforms
}

export const MIDI: Tool = {
    type: ToolType.OTHER,
    name: 'MIDI',
    version: null,
    icon: MIDI_IMG,
    description: null,
    url: 'https://en.wikipedia.org/wiki/MIDI',
    depend_tools: [],
    developed_platforms: Unity3D.developed_platforms
}

export const Cordova: Tool = {
    type: ToolType.OTHER,
    name: 'Cordova',
    version: null,
    icon: CORDOVA_IMG,
    description: null,
    url: 'https://cordova.apache.org/',
    depend_tools: [],
    developed_platforms: [ToolPlatform.MOBILE]
}

export const Electron: Tool = {
    type: ToolType.OTHER,
    name: 'Electron',
    version: null,
    icon: ELECTRON_IMG,
    description: null,
    url: 'https://www.electronjs.org/',
    depend_tools: [],
    developed_platforms: [ToolPlatform.DESKTOP]
}


export const TOOLS: Tool[] = [
    Ruby, Rails, Php, Symfony, Sylius, Javascript, Typescript, React, Hotwire, Dart, Flutter, Rust,
    CSS, SASS, Tailwind, Bootstrap,
    Webpack, RequireJS, Vite,
    Java, OpenGL, LWJGL, CSharp, Unity3D, GodotEngine
]

/**
 * @param types
 * @example
 *      filterToolByTypes(ToolType.FRAMEWORK | ToolType.LANGUAGE)
 */
export function filterToolByTypes(types: number) {
    TOOLS.filter((t) => (t.type.valueOf() & types) > 0)
}